# the P language

package Desktop::P;

require Exporter;
no encoding;
use utf8;

@ISA = qw(Exporter);

%EXPORT_TAGS = ( 'all' => [ 
 qw(error notice say decode
    flagp esc coalesce a wrap xwrap nul null
    log_msg log_err log_warn log_debug log_verbose
    human_size l10n mkdirhier dirname basename) ] );

@EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
@EXPORT = ( @{ $EXPORT_TAGS{'all'} } );
$VERSION = '0.6';

use strict;
use warnings;

#use Text::Unaccent;

#################################################################
# STRINGS
#################################################################

sub human_size {
  my ($size)=@_;
  
  if($size<1024) { return $size; }
  $size/=1024.0;
  if($size<1024) { return int($size+0.5)."K"; }
  $size/=1024.0;
  return sprintf("%.1fM",$size+0.05);
}

sub str_repeat { $_[0]x$_[1]; }

# text wrap
sub wrap {
  if(defined($_[1])) {
    return $_[0].$_[1].(defined($_[2])?$_[2]:'');
  } else {
    return '';
  }
}

# XML wrap
sub xwrap {
  my ($elem,$content,$attr)=@_;

  if(defined($content)) {
    return "<$elem>".$content."</$elem>";
  } else {
    return "<$elem>";
  }
}

sub decode {
  my ($expr,@a)=@_;
  my $def;
  if(($#a+1) & 1) { $def=pop(@a) };
  my %a=@a;
  return defined($a{$expr})?$a{$expr}:$def;
}

#################################################################
# NULLS n STUFF
#################################################################

sub coalesce {
  for my $i (@_) {
    return $i if defined $i;
  }
  return '';
}

#----------------------------------------------------------------

sub nul {
  if($_[0]) { return $_[0]; }
  return undef;
}

#----------------------------------------------------------------

sub null { undef; }

#----------------------------------------------------------------

sub isset { defined($_[0]) };

#################################################################
# string escaping
#################################################################

sub esc { 
  my ($to,$str)=@_;

  use URI::Escape;

  $to=lc($to);
  if($to eq "html" || $to eq "xml") {
    $str=~s/&/&amp;/go;
    $str=~s/</&lt;/go;
    $str=~s/>/&gt;/go;
    return $str;
  }
  if($to eq "attr") {
    $str=~s/&/&amp;/go;
    $str=~s/</&lt;/go;
    $str=~s/>/&gt;/go;
    $str=~s/"/&quot;/go;
#    $str=~s/'/&rsquo;/go;
    return $str;
  }
  elsif($to eq "url" || $to eq "uri") {
    $str=uri_escape($str,"^A-Za-z/");
    return $str;
  }
  elsif($to eq "c" || $to eq "javascript") {
    $str=~s/\n/\\n/g;
  }
  elsif($to eq "ascii" || $to eq "unac") {
    $str = unac_string("utf8",$str);
    return $str;
  }
  elsif($to eq "unix" || $to eq "file" || $to eq "name") {
#   $str=lc($str);

    $str = lc(unac_string("utf8",$str));

    # this seems to be necesary for MacOS...
    $str=~ s/%([0-9A-F][0-9A-F])/chr(hex($1))/oeg; # uri unescape

    # this seems to be necesary for Windows...
    $str=~ s/^.*\\//o; # basename

    $str=~ s/^\.*//;   # remove leading .'s
    $str=~ y/\\\//--/; # change \ and / to -
    $str=~ y/+-.a-zA-Z0-9/_/c; # change nonascii chars
    $str=~ s/_+/_/g; # compress
    $str=~ s/_$//; # trim
    return $str;	
  }
  elsif($to eq "wiki") {
    $str = unac_string("utf8",$str);
    $str=~ y/a-zA-Z0-9/_/c;
    $str=~ s/_+/_/g;
    $str=~ s/_(.)/uc($1)/ge;
    $str=~ s/_$//;
    return $str;
  }
  elsif($to eq "iso_date") {
    return iso_date($str);
  }

  # sh

  $str=~s/"/\\"/g;
  $str=~s/`/\\`/g;
  $str=qq{"$str"};

  return $str;
}

#################################################################
# logging functions (log_init,log_msg,log_warn,log_err)
#################################################################

sub notice {
  print "<p><b>NOTICE: $_[0]</b></p>\n";
}

sub log_msg {
  my ($level,$msg)=@_;
  if(1) {
    print "$level:$msg\n";
  }
};

#################################################################

our %ANSI_COLORS=(
		  CRE=>"^M\033[K",
		  NORMAL=>"\033[0;39m",
		  # RED: Failure or error message
		  RED=>"\033[1;31m",
		  # GREEN: Success message
		  GREEN=>"\033[1;32m",
		  # YELLOW: Descriptions
		  YELLOW=>"\033[1;33m",
		  # BLUE: System messages
		  BLUE=>"\033[1;34m",
		  # MAGENTA: Found devices or drivers
		  MAGENTA=>"\033[1;35m",
		  # CYAN: Questions
		  CYAN=>"\033[1;36m",
		  # BOLD WHITE: Hint
		  WHITE=>"\033[1;37m",
		 );

#################################################################

sub log_err {
  print $ANSI_COLORS{'RED'}."$_[0]".$ANSI_COLORS{'NORMAL'}."\n";
};

sub log_warn {
  print $ANSI_COLORS{'RED'}."$_[0]".$ANSI_COLORS{'NORMAL'}."\n";
};

sub log_verbose {
  print $ANSI_COLORS{'BLUE'}."$_[0]".$ANSI_COLORS{'NORMAL'}."\n";
};

sub log_debug {
  if($WDBI::debug{$_[0]} || $WDBI::debug) {
    print "$_[0]:$_[1]\n";
  }
};

sub log_init {
  print "$_[0]\n";
};

sub log_db {
  my ($name,$text,$args)=@_;

  my $ref="NULL";
  if($args->{'ref'}) { $ref=$args->{'ref'}; }

  my $query="
INSERT INTO wdbi_log
 (ctime,app,name,text,pid,ref)
VALUES
 (now(),?,?,?,?,?)
";

  query($query,$WDBI::app,$name,$text,$$,$ref);

};

sub iso_date {
  my ($time)=@_;
  my ($y,$m,$d);
  if(defined($time)) {
        ($y,$m,$d)=(localtime($time))[5,4,3];
  } else {
        ($y,$m,$d)=(localtime)[5,4,3];
  }
  sprintf("%04d-%02d-%02d",1900+$y,$m+1,$d);
}

sub error {
  print "Error: $_[0]\n";
}

sub a {
  my ($url,$query,$target,$label,$extra)=@_;

  $label=coalesce($label,$url);

  return ('<a href="'.$url.wrap('?',nul($query)).'"'.wrap(' target="',nul($target),'"').'>'.$label.'</a>');
}

sub nl { "\n"; }

sub l10n {
  my ($str,$lang)=@_;
  $lang=coalesce($lang,$WDBI::session{'lang'});

  my $s=$WDBI::l10n->{$lang}{$str};
  return defined($s)?$s:$str;
};

sub flagp { index($_[0],$_[1])>=0; }

sub mkdirhier {
  my $path=shift;
  system("/usr/bin/X11/mkdirhier ".esc("sh",$path));
}

sub dirname {
  my $path=shift;
  if(index($path,'/')>=0) {
    $path=~s|^(.*)/[^/]+$|$1|;
    return $path;
  } else {
    return undef;
  }
}

sub basename {
  my ($path)=@_;

  $path=~s|^.*/||;
  return $path;
}
